@@ -1251,6 +1251,7 @@ xref_meta(catch_with_backtrace(A, _, B), [A, B]).
 xref_meta(thread_create(A,_,_), [A]).
 xref_meta(thread_create(A,_),   [A]).
 xref_meta(thread_signal(_,A),   [A]).
+xref_meta(thread_idle(A,_),     [A]).
 xref_meta(thread_at_exit(A),    [A]).
 xref_meta(thread_initialization(A), [A]).
 xref_meta(engine_create(_,A,_), [A]).
@@ -1290,6 +1291,7 @@ xref_meta(prolog_listen(Ev,G),  [G+N]) :- event_xargs(Ev, N).
 xref_meta(prolog_listen(Ev,G,_),[G+N]) :- event_xargs(Ev, N).
 xref_meta(tnot(G),		[G]).
 xref_meta(not_exists(G),	[G]).
+xref_meta(with_tty_raw(G),	[G]).
 
                                         % XPCE meta-predicates
 xref_meta(pce_global(_, new(_)), _) :- !, fail.
@@ -1844,6 +1846,8 @@ prolog:no_autoload_module(library(record)).
 prolog:no_autoload_module(library(persistency)).
 prolog:no_autoload_module(library(pldoc)).
 prolog:no_autoload_module(library(settings)).
+prolog:no_autoload_module(library(debug)).
+prolog:no_autoload_module(library(plunit)).
 
 
 %!  process_requires(+Import, +Src)
@@ -2221,7 +2225,8 @@ open_include_file(Path, In, [Ref]) :-
     stream_property(Parent, encoding(Enc)),
     '$push_input_context'(xref_include),
     catch((   prolog:xref_open_source(Path, In)
-          ->  set_stream(In, encoding(Enc))
+          ->  catch(set_stream(In, encoding(Enc)),
+                    error(_,_), true)       % deal with non-file input
           ;   include_encoding(Enc, Options),
               open(Path, read, In, Options)
           ), E,
