@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2019, University of Amsterdam
+    Copyright (c)  1997-2020, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
     All rights reserved.
@@ -89,8 +89,7 @@ typedef struct
 		 *******************************/
 
 struct PL_global_data
-{ uintptr_t	heap_base;		/* heap-base rounded downwards */
-  pl_defaults_t	defaults;		/* system default settings */
+{ pl_defaults_t	defaults;		/* system default settings */
   pl_options_t	options;		/* command-line options */
   State		stateList;		/* list of loaded states */
   int		initialised;		/* Heap is initialised */
@@ -239,6 +238,7 @@ struct PL_global_data
     InitialiseHandle initialise_tail;
     PL_dispatch_hook_t dispatch_events; /* PL_dispatch_hook() */
 
+    unsigned int  signature;		/* Foreign predicate signature */
     int		  _loaded;		/* system extensions are loaded */
   } foreign;
 
@@ -659,8 +659,7 @@ struct PL_local_data
     AbortHandle _abort_tail;
 
     buffer	_discardable_buffer;	/* PL_*() character buffers */
-    buffer	_buffer_ring[BUFFER_RING_SIZE];
-    int		_current_buffer_id;
+    string_stack string_buffers;	/* PL_*() string buffers */
 
     int		SP_state;		/* For SICStus interface */
   } fli;
@@ -707,6 +706,7 @@ struct PL_local_data
     struct _thread_sig   *sig_tail;	/* Tail of signal queue */
     DefinitionChain local_definitions;	/* P_THREAD_LOCAL predicates */
     simpleMutex scan_lock;		/* Hold for asynchronous scans */
+    alert_channel alert;		/* How to alert the thread */
   } thread;
 #endif
 
