@@ -536,6 +536,7 @@ simple:
   return cref;
 }
 
+int acquired = 0;
 
 ClauseRef
 firstClause(Word argv, LocalFrame fr, Definition def, ClauseChoice chp ARG_LD)
@@ -554,9 +555,11 @@ firstClause(Word argv, LocalFrame fr, Definition def, ClauseChoice chp ARG_LD)
 			      &def->impl.clauses,
 			      &ctx
 			      PASS_LD);
+#define CHK_STATIC_RELOADING() (LD->gen_reload && false(def, P_DYNAMIC))
   DEBUG(CHK_SECURE, assert(!cref || !chp->cref ||
 			   visibleClause(chp->cref->value.clause,
-					 generationFrame(fr))));
+					 generationFrame(fr)) ||
+			   CHK_STATIC_RELOADING()));
   release_def(def);
 
   return cref;
@@ -2692,15 +2695,16 @@ sizeofClauseIndexes(Definition def)
   ClauseIndex *cip;
   size_t size = 0;
 
-  acquire_def(def);
   if ( (cip=def->impl.clauses.clause_indexes) )
-  { for(; *cip; cip++)
+  { acquire_def(def);
+    for(; *cip; cip++)
     { ClauseIndex ci = *cip;
 
       if ( ISDEADCI(ci) )
 	continue;
       size += sizeofClauseIndex(ci);
     }
+    release_def(def);
   }
 
   return size;
