@@ -39,8 +39,15 @@
 #define LD LOCAL_LD
 
 #ifdef __WINDOWS__
-#include <windows.h>
-#undef small
+  #include <windows.h>
+  #undef small
+  #include <intrin.h>
+  #ifdef _MSC_VER
+    #if SIZEOF_VOIDP == 8
+      #pragma intrinsic(_BitScanReverse64)
+    #endif
+    #pragma intrinsic(_BitScanReverse)
+  #endif
 #endif
 
 		 /*******************************
@@ -67,23 +74,27 @@ MSB(size_t i)
 { unsigned long index;
 #if SIZEOF_VOIDP == 8
   unsigned __int64 mask = i;
-  BitScanReverse64(&index, mask);
+  _BitScanReverse64(&index, mask);
 #else
   unsigned long mask = i;
-  BitScanReverse(&index, mask);
+  _BitScanReverse(&index, mask);
 #endif
 
   return index;
 }
 
+#if SIZEOF_VOIDP == 8
 #define HAVE_MSB64 1
 static inline int
 MSB64(int64_t i)
 { unsigned long index;
-  BitScanReverse64(&index, i);
+  _BitScanReverse64(&index, i);
   return index;
 }
+#endif
 
+#define MEMORY_BARRIER() MemoryBarrier()
+#define MEMORY_RELEASE() MemoryBarrier()
 #define MEMORY_BARRIER() MemoryBarrier()
 
 static inline size_t
@@ -111,6 +122,8 @@ __builtin_popcount(size_t sz)
 #endif
 
 #ifdef HAVE_GCC_ATOMIC
+#define MEMORY_ACQUIRE()	__atomic_thread_fence(__ATOMIC_ACQUIRE)
+#define MEMORY_RELEASE()	__atomic_thread_fence(__ATOMIC_RELEASE)
 #define MEMORY_BARRIER()	__atomic_thread_fence(__ATOMIC_SEQ_CST)
 #endif
 
@@ -220,6 +233,8 @@ MSB64(int64_t i)
 
 #ifndef MEMORY_BARRIER
 #define MEMORY_BARRIER() (void)0
+#define MEMORY_ACQUIRE() (void)0
+#define MEMORY_RELEASE() (void)0
 #endif
 
 		 /*******************************
@@ -376,50 +391,6 @@ lookupDefinition(functor_t f, Module m)
 }
 
 
-static inline code
-fetchop(Code PC)
-{ code op = decode(*PC);
-
-  if ( unlikely(op == D_BREAK) )
-    op = decode(replacedBreak(PC));
-
-  return op;
-}
-
-
-static inline code			/* caller must hold the L_BREAK lock */
-fetchop_unlocked(Code PC)
-{ code op = decode(*PC);
-
-  if ( unlikely(op == D_BREAK) )
-    op = decode(replacedBreakUnlocked(PC));
-
-  return op;
-}
-
-
-static inline Code
-stepPC(Code PC)
-{ code op = fetchop(PC++);
-
-  if ( unlikely(codeTable[op].arguments == VM_DYNARGC) )
-    return stepDynPC(PC, &codeTable[op]);
-  else
-    return PC + codeTable[op].arguments;
-}
-
-
-static inline Code
-stepPC_unlocked(Code PC)
-{ code op = fetchop_unlocked(PC++);
-
-  if ( unlikely(codeTable[op].arguments == VM_DYNARGC) )
-    return stepDynPC(PC, &codeTable[op]);
-  else
-    return PC + codeTable[op].arguments;
-}
-
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Mark() sets LD->mark_bar, indicating  that   any  assignment  above this
 value need not be trailed.
@@ -638,26 +609,34 @@ normally 0). longToPointer() does the inverse operation.
 
 static inline uintptr_t
 pointerToInt(void *ptr)
-{ uintptr_t p   = (uintptr_t) ptr;
+{
+#if SIZEOF_VOIDP == 8
+  return (uintptr_t) ptr;
+#else
+  uintptr_t p   = (uintptr_t) ptr;
   uintptr_t low = p & 0x3L;
 
-  p -= GD->heap_base;
   p >>= 2;
   p |= low<<(sizeof(uintptr_t)*8-2);
 
   return p;
+#endif
 }
 
 
 static inline void *
 intToPointer(uintptr_t p)
-{ uintptr_t low = p >> (sizeof(uintptr_t)*8-2);
+{
+#if SIZEOF_VOIDP == 8
+  return (void*) p;
+#else
+  uintptr_t low = p >> (sizeof(uintptr_t)*8-2);
 
   p <<= 2;
   p |= low;
-  p += GD->heap_base;
 
   return (void *) p;
+#endif
 }
 
 #endif /*PL_INLINE_H_INCLUDED*/
