@@ -34,6 +34,7 @@
 */
 
 #include "pl-incl.h"
+#include "pl-comp.h"
 #include "os/pl-ctype.h"
 #include "os/pl-cstack.h"
 #include "pl-inline.h"
@@ -1001,7 +1002,7 @@ messageToString(term_t msg)
 
     PL_put_term(av+0, msg);
     rc = (PL_call_predicate(MODULE_system, PL_Q_NODEBUG, pred, av) &&
-	  PL_get_chars(av+1, &s, CVT_ALL|BUF_RING));
+	  PL_get_chars(av+1, &s, CVT_ALL|BUF_STACK));
     PL_discard_foreign_frame(fid);
 
     return rc ? s : (char*)NULL;
@@ -1796,6 +1797,17 @@ tracemode(int doit, int *old)
 }
 
 
+static int
+have_space_for_debugging(void)
+{ GET_LD
+
+  return ( usedStack(local) +
+	   usedStack(global) +
+	   usedStack(trail) +
+	   100000*sizeof(void*) < LD->stacks.limit );
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Enable the tracer if we have a safe amount of available space. This is
 used to start tracing uncaught overflow exceptions.
@@ -1803,13 +1815,9 @@ used to start tracing uncaught overflow exceptions.
 
 int
 trace_if_space(void)
-{ GET_LD
-  int trace;
+{ int trace;
 
-  if ( usedStack(local) +
-       usedStack(global) +
-       usedStack(trail) +
-       100000*sizeof(void*) < LD->stacks.limit )
+  if ( have_space_for_debugging() )
   { trace = TRUE;
     tracemode(trace, NULL);
 
@@ -1857,7 +1865,8 @@ debugmode(debug_type doit, debug_type *old)
 
   if ( debugstatus.debugging != doit )
   { if ( doit )
-    { if ( !enlargeMinFreeStacks(8*1024*SIZEOF_VOIDP,
+    { if ( have_space_for_debugging() &&
+	   !enlargeMinFreeStacks(8*1024*SIZEOF_VOIDP,
 				 8*1024*SIZEOF_VOIDP,
 				 8*1024*SIZEOF_VOIDP
 				 PASS_LD) )
